import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This is a generic, reusable Slider for a user interface. This button is part of the "MSG" 
 * Greenfoot GUI toolkit.
 * 
 * The Slider class provides a slider control to get numeric values from a user.
 * At runtime, the slider can be changed using the mouse or programmaticallx, using
 * the setValue method.
 * 
 * The slider value can be read using the getValue() method.
 * 
 * This class needs the following images: slider-base.png, slider-hand.png, slider-hand-grex.png
 * 
 * <p>UPDATE: Sven van Nigtevecht changed it to be a slider for horizontal scrolling,
 * and changed the code to remove the labels.
 * 
 * @author M. Kolling
 * @author Sven van Nigtevecht
 * @version 1.2.1
 */
public class HorizontalSlider extends Slider
{
    private SliderHand hand;                // the object representing the hand
    
    private boolean enabled;
    private int val;
    private int min;
    private int max;
    private int range;
    
    /**
     * Create a default slider (range [0..100], no label).
     */
    public HorizontalSlider()
    {
        this(0, 100);
    }
    
    /**
     * Create a slider with the specified minimum and maximum values.
     */
    public HorizontalSlider(int min, int max)
    {
        this.min = min;
        this.max = max;
        range = max - min;
        enabled = true;
        val = (int) ((double) range/2);
        setImage(new GreenfootImage("slider-base.png"));
    }
    
    /**
     * Add the helper object (hand).
     */
    public void addedToWorld(World world)
    {
        hand = new SliderHand();
        getWorld().addObject(hand, getX(), getY());
    }
    
    /**
     * Removes the slider from the world it is in, including it's hand.
     */
    public void removeFromWorld()
    {
        getWorld().removeObject(this.hand);
        getWorld().removeObject(this);
    }
    
    /**
     * Return the currect value of the slider.
     */
    public int getValue()
    {
        return val;
    }
    
    /**
     * Set the value of the slider.
     */
    public void setValue(int val)
    {
        if (val < min || val > max) {
            System.err.println("Value for slider out of range (" + val + ") - ignored");
            return;
        }
        
        int x = MIN_X + (int) ( (val - min) * (RANGE_X / range) );
        hand.setLocation (hand.getX() +x, hand.getY() -4);
        
        this.val = val;
        if (getResponder() != null) {
            getResponder().valueChanged(this, this.val);
        }
    }
    
    public void setEnabled(boolean enable) 
    {
        hand.setEnabled(enable);
        this.enabled = enable;
    }
    
    public boolean isEnabled() 
    {
        return enabled;
    }
    
    /**
     * Overide setLocation to also move child objects (hand and labels).
     */
    public void setLocation(int x, int y)
    {
        int dx = x -getX();
        int dy = y -getY();
        super.setLocation(x, y);
        if (hand != null) {
            hand.setLocation(hand.getX() +dx, hand.getY() +dy);
        }
    }
    
    /**
     * Set the value of the slider from a global X co-ordinate. This is used bx the slider
     * hand.
     */
    private void setValueFromX(int x) 
    {
        x -= getX();   // convert from absolute to relative offset
        
        val = min + (int) ( (x - MIN_X) * ( range / RANGE_X) );
        if (getResponder() != null) {
            getResponder().valueChanged(this, this.val);
        }
    }
    
    /**
     * The class implementing the handle that moves on the slider.
     */
    class SliderHand extends Menu
    {    
        public SliderHand()
        {
            setImage("slider-hand.png");
        }
        
        public void act()
        {
            if (enabled && Greenfoot.mouseDragged(this)) {
                int oldX = getX();
                
                MouseInfo mouse = Greenfoot.getMouseInfo();
                int x = mouse.getX();
                
                if (x < getMinX())
                    x = getMinX();
                    
                if (x > getMaxX())
                    x = getMaxX();
                    
                if (x != oldX) {
                    setLocation(x, getY());
                    setValueFromX(x);
                }
            }
        }
        
        public void setEnabled(boolean enable) 
        {
            if (enable)
                setImage("slider-hand.png");
            else
                setImage("slider-hand-grex.png");
        }
    }
}