import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Point;
import java.util.List;

/**
 * This world contains the main part of the scrolling system.
 * 
 * Here you can also limit the world by changing the variables WORLD_WIDTH and WORLD_HEIGHT using the constructor of the class (see ExampleWorld).
 * You can create scrolling worlds in every screen size (the size of the frame) and every scrolling size (the real size of the world).
 * 
 * The GreenfootImage textur is the background image that is used for the scrolling background. You can use any image you want instead of this. 
 * You just have to call the method setScrollingBackground in the constructor of your scrolling world (see ExampleWorld) with the GreenfootImage you want to use for the background.
 * 
 * The scenario will automaticly create a example world. You can delete this world and use another subclass of this world if you want to use the scrolling system in your scenario.
 * 
 * 
 * You shouldn't change the code of this class because that will probably cause problems in the scrolling system.
 */
public abstract class SlideBarScrollingWorld extends World
{
    //A maximum size of the Scrolling World. If the value for width or height is 0 the world is infinite in this direction.
    //The variables are final so they have to be set before compiling the game and can't be set while executing the game.
    public static int WORLD_WIDTH;
    public static int WORLD_HEIGHT;
    
    protected int totalXMovement = 0;
    protected int totalYMovement = 0;
    
    protected boolean bounded;
    
    protected VerticalSlider verticalSlideBar;
    protected HorizontalSlider horizontalSlideBar;
    
    //This image is used as the background image of the scrolling world.
    //If you want to set it to another image use the method setScrollingBackground(GreenfootImage).
    protected GreenfootImage textur;
    
    private MouseInfo mouse;
    
    /**
     * Create a new MouseScrollingWorld of the given screen size, scrolling size, cell size and boundment.
     * 
     * @param screenWidth
     *      The width of the frame that shows a part of the scrolling world.
     * 
     * @param screenHeight
     *      The height of the frame that shows a part of the scrolling world.
     * 
     * @param scrollingWidth
     *      The width of the whole scrolling system.
     * 
     * @param scrollingHeight
     *      The height of the whole scrolling system.
     */
    public SlideBarScrollingWorld(int screenWidth, int screenHeight, int scrollingWidth, int scrollingHeight) throws IllegalArgumentException {
        this(screenWidth, screenHeight, 1, scrollingWidth, scrollingHeight, false);
    }
    /**
     * Create a new MouseScrollingWorld of the given screen size, scrolling size, cell size and boundment.
     * 
     * @param screenWidth
     *      The width of the frame that shows a part of the scrolling world.
     * 
     * @param screenHeight
     *      The height of the frame that shows a part of the scrolling world.
     * 
     * @param scrollingWidth
     *      The width of the whole scrolling system.
     * 
     * @param scrollingHeight
     *      The height of the whole scrolling system.
     * 
     * @param bounded
     *      If true the actors can't be moved out of the scrolling world but the actors are always able to be moved out of the screen.
     */
    public SlideBarScrollingWorld(int screenWidth, int screenHeight, int scrollingWidth, int scrollingHeight, boolean bounded) throws IllegalArgumentException {
        this(screenWidth, screenHeight, 1, scrollingWidth, scrollingHeight, bounded);
    }
    /**
     * Create a new MouseScrollingWorld of the given screen size, scrolling size, cell size and boundment.
     * 
     * @param screenWidth
     *      The width of the frame that shows a part of the scrolling world.
     * 
     * @param screenHeight
     *      The height of the frame that shows a part of the scrolling world.
     * 
     * @param cellSize
     *      The size of each cell in the scrolling world.
     * 
     * @param scrollingWidth
     *      The width of the whole scrolling system.
     * 
     * @param scrollingHeight
     *      The height of the whole scrolling system.
     */
    public SlideBarScrollingWorld(int screenWidth, int screenHeight, int cellSize, int scrollingWidth, int scrollingHeight) throws IllegalArgumentException {
        this(screenWidth, screenHeight, cellSize, scrollingWidth, scrollingHeight, false);
    }
    /**
     * Create a new MouseScrollingWorld of the given screen size, scrolling size, cell size and boundment.
     * 
     * @param screenWidth
     *      The width of the frame that shows a part of the scrolling world.
     * 
     * @param screenHeight
     *      The height of the frame that shows a part of the scrolling world.
     * 
     * @param cellSize
     *      The size of each cell in the scrolling world.
     * 
     * @param scrollingWidth
     *      The width of the whole scrolling system.
     * 
     * @param scrollingHeight
     *      The height of the whole scrolling system.
     * 
     * @param bounded
     *      If true the actors can't be moved out of the scrolling world but the actors are always able to be moved out of the screen.
     */
    public SlideBarScrollingWorld(int screenWidth, int screenHeight, int cellSize, int scrollingWidth, int scrollingHeight, boolean bounded) throws IllegalArgumentException {
        super(screenWidth, screenHeight, cellSize, false);
        if (scrollingWidth < screenWidth || scrollingHeight < screenHeight) {
            throw new IllegalArgumentException("scrollingWidth(" + scrollingWidth + ") and scrollingHeight(" + scrollingHeight + ") must be at leased as big as screenWidth / screenHeight.");
        }
        WORLD_WIDTH = scrollingWidth;
        WORLD_HEIGHT = scrollingHeight;
        this.bounded = bounded;
        setPaintOrder(Menu.class);
        verticalSlideBar = new VerticalSlider(0, 1000);
        horizontalSlideBar = new HorizontalSlider(0, 1000);
        addSlideBars();
    }
    
    /**
     * This is the act method of the world that ensures that the mouse scrolling system works.
     * Because this method is final it can't be overwritten. So you can't use an act method in your subclass. But the abstract method run() will 
     * always be called from the act mehtod so you can use the run mehtod as your new act method. 
     * It works the same way. All the code within the method will be executed every act cycle.
     */
    public final void act() {
        checkSlideBars();
        run();
    }
    
    /**
     * This abstract method has to be overwritten in your subclass and can be used the same way as an act mehtod.
     * It'll execute the code every act cycle.
     */
    public abstract void run();
    
    /**
     * Checks the value of the slide bars and scrolls the world.
     */
    protected final void checkSlideBars() {
        int currentXMovement = totalXMovement;
        int currentYMovement = totalYMovement;
        totalXMovement = (int) (((double) -WORLD_WIDTH/1000 * horizontalSlideBar.getValue()) + WORLD_WIDTH/2);
        totalYMovement = (int) (((double) -WORLD_HEIGHT/1000 * verticalSlideBar.getValue()) + WORLD_HEIGHT/2);
        scrollWorldObjects(totalXMovement - currentXMovement, totalYMovement - currentYMovement);
    }
    
    /**
     * This mehtod will make your objects move by the scrolling system.
     * 
     * @param scrollingX
     *      The width all actors have to be moved.
     * 
     * @param scrollingY
     *      The height all actors have to be moved.
     */
    public final void scrollWorldObjects(int scrollingX, int scrollingY) {
        List<Actor> actors = getObjects(Actor.class);
        for (Actor actor : actors) {
            if (actor instanceof Menu || actor instanceof FixedObject) {
                ;
            }
            else {
                actor.setLocation(actor.getX() + scrollingX, actor.getY() + scrollingY);
            }
        }
        createTextur();
    }
    
    /**
     * Creates a moving textur on the background image of the world.
     */
    protected final void createTextur() {
        int x;
        int y;
        if (totalXMovement > 0) {
            for (x = totalXMovement; x > 0; x -= textur.getWidth()) {
                ;
            }
        }
        else {
            for (x = totalXMovement; x < 0; x += textur.getWidth()) {
                ;
            }
            x -= textur.getWidth();
        }
        if (totalYMovement > 0) {
            for (y = totalYMovement; y > 0; y -= textur.getHeight()) {
                ;
            }
        }
        else {
            for (y = totalYMovement; y < 0; y += textur.getHeight()) {
                ;
            }
            y -= textur.getHeight();
        }
        getBackground().clear();
        for (int i = x; i < getWidth(); i += textur.getWidth()) {
            for (int j = y; j < getHeight(); j += textur.getHeight()) {
                getBackground().drawImage(textur, i, j);
            }
        }
    }
    
    /**
     * Scroll the world to the given scrolling point.
     * 
     * @param x
     *      The x location of the scrolling point.
     * 
     * @param y
     *      The y location of the scrolling point.
     */
    public final void scrollToPosition(double x, double y) {
        int scrollingX;
        int scrollingY;
        if (x == -totalXMovement && y == -totalYMovement) {
            return;
        }
        scrollingX = (int) (-totalXMovement - x);
        scrollingY = (int) (-totalYMovement - y);
        totalXMovement = (int) -x;
        totalYMovement = (int) -y;
        scrollWorldObjects(scrollingX, scrollingY);
        System.out.println(scrollingX + " " + scrollingY);
    }
    /**
     * Scroll the world to the given scrolling point.
     * 
     * @param scrollingPoint
     *      The scrolling point the system is moved to.
     */
    public final void scrollToPosition(Point scrollingPoint) {
        scrollToPosition(scrollingPoint.getX(), scrollingPoint.getY());
    }
    
    /**
     * Remove all objects that currently are in the world.
     */
    public void removeAllObjects() {
        removeObjects(getObjects(null));
    }
    
    /**
     * Remove all objects that currently are in the world with the exception of the actor given as parameter.
     * 
     * @param actor
     *      The only object that will not be removed from the world.
     */
    public void removeAllObjectsBut(Actor actor) {
        List<Actor> allActors = getObjects(null);
        for (Actor a : allActors) {
            if (!a.equals(actor)) {
                removeObject(a);
            }
        }
    }
    
    /**
     * Adds the slide bars to the world that are used for the scrolling.
     */
    public final void addSlideBars() {
        addObject(verticalSlideBar, getWidth() - 30, getHeight()/2);
        addObject(horizontalSlideBar, getWidth()/2, getHeight() -30);
    }
    
    /**
     * Check whether the world is bounded or not.
     * 
     * @return
     *      Returns true if the world is bounded. False if not.
     */
    public boolean isBounded() {
        return bounded;
    }
    
    /**
     * Change the background image of the scrolling world to the given image.
     * 
     * @param bgImage
     *      The new background image.
     */
    public void setScrollingBackground(GreenfootImage bgImage) {
        textur = bgImage;
        createTextur();
    }
    
    /**
     * Returns the width of to scrolling world. (0 => infinite).
     * 
     * @return
     *      The width of the scrolling system. If 0 is returned the world width is infinite.
     */
    public int getScrollingWidth() {
        return WORLD_WIDTH;
    }
    /**
     * Returns the height of to scrolling world. (0 => infinite).
     * 
     * @return
     *      The height of the scrolling system. If 0 is returned the world height is infinite.
     */
    public int getScrollingHeight() {
        return WORLD_HEIGHT;
    }
    
    /**
     * Get the total movement in x direction.
     * 
     * @return
     *      The total movement in x direction the scrolling actor has covered.
     */
    public int getTotalXMovement() {
        return totalXMovement;
    }
    /**
     * Get the total movement in y direction.
     * 
     * @return
     *      The total movement in y direction the scrolling actor has covered.
     */
    public int getTotalYMovement() {
        return totalYMovement;
    }
}