import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This is a generic, reusable Slider for a user interface. This button is part of the "MSG" 
 * Greenfoot GUI toolkit.
 * 
 * The Slider class provides a slider control to get numeric values from a user.
 * At runtime, the slider can be changed using the mouse or programmatically, using
 * the setValue method.
 * 
 * The slider value can be read using the getValue() method.
 * 
 * This class needs the following images: slider-base.png, slider-hand.png, slider-hand-grey.png
 * 
 * <p>UPDATE: Sven van Nigtevecht changed it to be a slider for vertical scrolling,
 * and changed the code to remove the labels.
 * 
 * @author M. Kolling
 * @author Sven van Nigtevecht
 * @version 1.2.1
 */
public class VerticalSlider extends Slider
{
    private SliderHand hand;                // the object representing the hand
    
    private boolean enabled;
    private int val;
    private int min;
    private int max;
    private int range;
    
    /**
     * Create a default slider (range [0..100], no label).
     */
    public VerticalSlider()
    {
        this(0, 100);
    }
    
    /**
     * Create a slider with the specified minimum and maximum values.
     */
    public VerticalSlider(int min, int max)
    {
        this.min = min;
        this.max = max;
        range = max - min;
        enabled = true;
        val = (int) ((double) range/2);
        setImage(new GreenfootImage("slider-base.png"));
        setRotation(270);
    }
    
    /**
     * Add the helper objects (hand and labels).
     */
    public void addedToWorld(World world)
    {
        hand = new SliderHand();
        getWorld().addObject(hand, getX() +4, getY());
    }
    
    /**
     * Removes the slider from the world it is in, including it's hand.
     */
    public void removeFromWorld()
    {
        getWorld().removeObject(this.hand);
        getWorld().removeObject(this);
    }
    
    /**
     * Return the currect value of the slider.
     */
    public int getValue()
    {
        return val;
    }
    
    /**
     * Set the value of the slider.
     */
    public void setValue(int val)
    {
        if (val < min || val > max) {
            System.err.println("Value for slider out of range (" + val + ") - ignored");
            return;
        }
        
        int y = MIN_Y + (int) ( (val - min) * (RANGE_Y / range) );
        hand.setLocation (hand.getX() +4, hand.getY() +y);
        
        this.val = val;
        if (this.getResponder() != null) {
            this.getResponder().valueChanged(this, this.val);
        }
    }
    
    public void setEnabled(boolean enable) 
    {
        hand.setEnabled(enable);
        this.enabled = enable;
    }
    
    public boolean isEnabled() 
    {
        return enabled;
    }
    
    /**
     * Overide setLocation to also move child objects (hand and labels).
     */
    public void setLocation(int x, int y)
    {
        int dx = x -getX();
        int dy = y -getY();
        super.setLocation(x, y);
        if (hand != null) {
            hand.setLocation(hand.getX() +dx, hand.getY() +dy);
        }
    }
    
    /**
     * Set the value of the slider from a global X co-ordinate. This is used by the slider
     * hand.
     */
    private void setValueFromY(int y) 
    {
        y -= getY();   // convert from absolute to relative offset
        
        val = min + (int) ( (y - MIN_Y) * ( range / RANGE_Y) );
        if (this.getResponder() != null) {
            this.getResponder().valueChanged(this, this.val);
        }
    }
    
    /**
     * The class implementing the handle that moves on the slider.
     */
    class SliderHand extends Menu
    {    
        public SliderHand()
        {
            setImage("slider-hand.png");
            setRotation(90);
        }
        
        public void act()
        {
            if (enabled && Greenfoot.mouseDragged(this)) {
                int oldY = getX();
                
                MouseInfo mouse = Greenfoot.getMouseInfo();
                int y = mouse.getY();
                
                if (y < getMinY())
                    y = getMinY();
                    
                if (y > getMaxY())
                    y = getMaxY();
                    
                if (y != oldY) {
                    setLocation(getX(), y);
                    setValueFromY(y);
                }
            }
        }
        
        public void setEnabled(boolean enable) 
        {
            if (enable)
                setImage("slider-hand.png");
            else
                setImage("slider-hand-grey.png");
        }
    }
}